#!/usr/bin/env python3

# Wrapper for 'swaylock' & 'gtklock' commands to use them with a random image, local of downloaded from unsplash.com.
# All the setting come from the 'nwg-shell-config' 'settings' data file.

import os
import random
import subprocess
import sys

from nwg_shell_config.tools import get_data_dir, temp_dir, load_json, gtklock_module_path, \
    playerctl_metadata, eprint

try:
    import requests
except ModuleNotFoundError:
    eprint("python-requests module not found, terminating")
    sys.exit(1)

config_home = os.getenv('XDG_CONFIG_HOME') if os.getenv('XDG_CONFIG_HOME') else os.path.join(os.getenv("HOME"),
                                                                                             ".config/")
data_dir = get_data_dir()
tmp_dir = temp_dir()

if os.getenv("SWAYSOCK"):
    settings = load_json(os.path.join(data_dir, "settings"))
elif os.getenv("HYPRLAND_INSTANCE_SIGNATURE"):
    settings = load_json(os.path.join(data_dir, "settings-hyprland"))
else:
    eprint("This program only works on sway or Hyprland, terminating.")
    sys.exit(1)

preset = load_json(
    os.path.join(data_dir, settings["panel-preset"])) if "panel-preset" in settings and "panel-preset" else {}

pid = os.getpid()

defaults = {
    "panel-preset": "preset-0",
    "lockscreen-use-settings": True,
    "lockscreen-locker": "gtklock",  # swaylock | gtklock
    "lockscreen-background-source": "local",  # unsplash | local
    "lockscreen-custom-cmd": "",
    "lockscreen-timeout": 1200,
    "sleep-cmd": 'swaymsg "output * dpms off"',
    "sleep-timeout": 1800,
    "resume-cmd": 'swaymsg "output * dpms on"',
    "after-resume": 'swaymsg "output * enable"',
    "before-sleep": "",
    "backgrounds-custom-path": "",
    "backgrounds-use-custom-path": False,
    "background-dirs": ["/usr/share/backgrounds/nwg-shell"],
    "background-dirs-once-set": False,
    "unsplash-width": 1920,
    "unsplash-height": 1080,
    "unsplash-keywords": ["nature", "water", "landscape"],
    "wallhaven-api-key": "",
    "wallhaven-ratio": "16x9,16x10",
    "wallhaven-atleast": "1920x1080",
    "wallhaven-tags": ["nature", "landscape"],
    "gtklock-disable-input-inhibitor": False,
    "gtklock-idle-timeout": 10,
    "gtklock-logout-command": "swaymsg exit",
    "gtklock-playerctl": False,
    "gtklock-powerbar": False,
    "gtklock-poweroff-command": "systemctl -i poweroff",
    "gtklock-reboot-command": "systemctl reboot",
    "gtklock-suspend-command": "systemctl suspend",
    "gtklock-time-format": "%H:%M:%S",
    "gtklock-userinfo": False,
    "gtklock-userswitch-command": ""
}

preset_defaults = {
    "gtklock-userinfo-round-image": True,
    "gtklock-userinfo-vertical-layout": False,
    "gtklock-userinfo-under-clock": False,
    "gtklock-powerbar-show-labels": False,
    "gtklock-powerbar-linked-buttons": False,
    "gtklock-playerctl-art-size": 64,
    "gtklock-playerctl-position": "top-right",
    "gtklock-playerctl-show-hidden": True
}


def set_remote_wallpaper():
    wallpaper_path = os.path.join(data_dir, "wallpaper.jpg")
    try:
        load_wallhaven_image(wallpaper_path)
        if settings["lockscreen-locker"] == "swaylock":
            subprocess.Popen('swaylock -i {}'.format(wallpaper_path), shell=True)
        elif settings["lockscreen-locker"] == "gtklock":
            eprint('{} -S -H -T 10 -b {}'.format(gtklock_command(), wallpaper_path))
            subprocess.Popen('{} -S -H -T 10 -b {}'.format(gtklock_command(), wallpaper_path), shell=True)

    except Exception as e:
        eprint("Failed loading image from wallhaven.cc :/")
        set_local_wallpaper()


def load_wallhaven_image(path):
    api_key = settings['wallhaven-api-key'] if settings["wallhaven-api-key"] else None
    api_key_status = "set" if settings['wallhaven-api-key'] else "unset"
    tags = " ".join(settings['wallhaven-tags']) if settings['wallhaven-tags'] else "nature, landscape"
    ratios = settings['wallhaven-ratio'] if settings['wallhaven-ratio'] else "16x9,16x10"
    atleast = settings["wallhaven-atleast"] if settings["wallhaven-atleast"] else "1920x1080"
    print(
        f"Fetching random image from wallhaven.cc, tags: '{tags}', ratios: '{ratios}', atleast: '{atleast}', API key: {api_key_status}")
    # Wallhaven API endpoint
    url = "https://wallhaven.cc/api/v1/search"

    # Parameters for the API request
    params = {
        "q": tags,
        "ratios": ratios,
        "atleast": atleast,
        "sorting": "random"
    }
    if api_key:
        params["apikey"] = api_key

    # Make the API request
    response = requests.get(url, params=params)

    # Get the image URL from the response
    if response.status_code == 200:
        image_data = response.json()
        image_url = image_data["data"][0]["path"]

        for key in image_data["data"][0]:
            print(f"{key}: {image_data['data'][0][key]}")

        # Download the image
        image_response = requests.get(image_url)

        if image_response.status_code == 200:
            # Save the image locally
            with open(path, "wb") as file:
                file.write(image_response.content)
            print(f"Wallhaven image saved as {path}")
        else:
            eprint("Failed to download Wallhaven image")
    else:
        eprint("Failed to fetch Wallhaven image")


def set_local_wallpaper():
    paths = []
    dirs = settings["background-dirs"].copy()
    if settings["backgrounds-use-custom-path"] and settings["backgrounds-custom-path"]:
        dirs.append(settings["backgrounds-custom-path"])
    for d in dirs:
        for item in os.listdir(d):
            if os.path.isfile(os.path.join(d, item)):
                paths.append(os.path.join(d, item))
    if len(paths) > 0:
        p = paths[random.randrange(len(paths))]
        if settings["lockscreen-locker"] == "swaylock":
            subprocess.Popen('swaylock -i {}'.format(p), shell=True)

        elif settings["lockscreen-locker"] == "gtklock":
            eprint(
                '{} -S -H -T {} -b {}'.format(gtklock_command(), settings["gtklock-idle-timeout"], p))
            subprocess.Popen(
                '{} -S -H -T {} -b {}'.format(gtklock_command(), settings["gtklock-idle-timeout"], p), shell=True)
    else:
        print("No image paths found")

        if settings["lockscreen-locker"] == "swaylock":
            subprocess.Popen('exec swaylock -f', shell=True)

        elif settings["lockscreen-locker"] == "gtklock":
            subprocess.Popen('exec gtklock -d', shell=True)

    sys.exit(0)


def gtklock_command():
    gtklock_cmd = "gtklock"

    if settings["gtklock-disable-input-inhibitor"]:
        gtklock_cmd += " -i"

    # userinfo module
    if settings["gtklock-userinfo"]:
        gtklock_cmd += " -m {}".format(gtklock_module_path("userinfo"))

        # optional userinfo module arguments
        if not preset["gtklock-userinfo-round-image"]:
            gtklock_cmd += " --no-round-image"
        if not preset["gtklock-userinfo-vertical-layout"]:
            gtklock_cmd += " --horizontal-layout"
        if preset["gtklock-userinfo-under-clock"]:
            gtklock_cmd += " --under-clock"

    # powerbar module
    if settings["gtklock-powerbar"]:
        gtklock_cmd += " -m {}".format(gtklock_module_path("powerbar"))

        # optional powerbar module arguments
        if preset["gtklock-powerbar-show-labels"]:
            gtklock_cmd += " --show-labels"
        if preset["gtklock-powerbar-linked-buttons"]:
            gtklock_cmd += " --linked-buttons"
        # It seems a bit counter-intuitive, but to turn one of gtklock poweroff module commands off,
        # we need to pass the empty string, like `--suspend-command`.
        # See: https://github.com/jovanlanik/gtklock-powerbar-module/issues/1#issuecomment-1287509956
        if settings["gtklock-reboot-command"]:
            gtklock_cmd += " --reboot-command '{}'".format(settings["gtklock-reboot-command"])
        else:
            gtklock_cmd += " --reboot-command ''"
        if settings["gtklock-poweroff-command"]:
            gtklock_cmd += " --poweroff-command '{}'".format(settings["gtklock-poweroff-command"])
        else:
            gtklock_cmd += " --poweroff-command ''"
        if settings["gtklock-suspend-command"]:
            gtklock_cmd += " --suspend-command '{}'".format(settings["gtklock-suspend-command"])
        else:
            gtklock_cmd += " --suspend-command ''"

        if settings["gtklock-logout-command"]:
            gtklock_cmd += " --logout-command '{}'".format(settings["gtklock-logout-command"])
        else:
            gtklock_cmd += " --logout-command ''"

        if settings["gtklock-userswitch-command"]:
            gtklock_cmd += " --userswitch-command '{}'".format(settings["gtklock-userswitch-command"])
        else:
            gtklock_cmd += " --userswitch-command ''"

    # playerctl module
    # Don't show if playerctl_metadata() empty.
    # https://github.com/jovanlanik/gtklock-playerctl-module/issues/4
    if settings["gtklock-playerctl"] and playerctl_metadata():
        gtklock_cmd += " -m {}".format(gtklock_module_path("playerctl"))

        # optional playerctl module arguments
        if preset["gtklock-playerctl-show-hidden"]:
            gtklock_cmd += " --show-hidden"
        if "gtklock-playerctl-art-size" in preset:
            gtklock_cmd += " --art-size {}".format(preset["gtklock-playerctl-art-size"])
        if "gtklock-playerctl-position":
            gtklock_cmd += " --position '{}'".format(preset["gtklock-playerctl-position"])

    if settings["gtklock-time-format"]:
        gtklock_cmd += " --time-format '{}'".format(settings["gtklock-time-format"])

    # gtklock style sheets
    if settings["panel-preset"]:
        gtklock_config_dir = os.path.join(config_home, "gtklock")
        css_file = os.path.join(gtklock_config_dir, "{}.css".format(settings["panel-preset"]))

        if os.path.isfile(css_file):
            gtklock_cmd += " -s {}".format(css_file)

    return gtklock_cmd


def main():
    global defaults
    for key in defaults:
        if key not in settings:
            settings[key] = defaults[key]

    global preset_defaults
    for key in preset_defaults:
        if key not in preset:
            preset[key] = preset_defaults[key]

    if settings["lockscreen-custom-cmd"]:
        subprocess.Popen('exec {}'.format(settings["lockscreen-custom-cmd"]), shell=True)
        sys.exit(0)

    if settings["lockscreen-background-source"] == "wallhaven.cc":
        set_remote_wallpaper()

    elif settings["lockscreen-background-source"] == "local":
        set_local_wallpaper()


if __name__ == '__main__':
    main()



import json
import os
import shutil
import socket
import subprocess
import sys
import tarfile
from shutil import copy2

from geopy.geocoders import Nominatim


def get_config_home():
    config_home = os.getenv('XDG_CONFIG_HOME') if os.getenv('XDG_CONFIG_HOME') else os.path.join(
        os.getenv("HOME"), ".config/")
    return config_home


def get_data_home():
    d_home = os.getenv('XDG_DATA_HOME') if os.getenv('XDG_DATA_HOME') else os.path.join(
        os.getenv("HOME"), ".local/share")
    return d_home


def get_data_dir():
    data_dir = ""
    home = os.getenv("HOME")
    xdg_data_home = os.getenv("XDG_DATA_HOME")

    if xdg_data_home:
        data_dir = os.path.join(xdg_data_home, "nwg-shell-config/")
    else:
        if home:
            data_dir = os.path.join(home, ".local/share/nwg-shell-config/")

    if not os.path.isdir(data_dir):
        print("Creating '{}'".format(data_dir))
        os.makedirs(data_dir, exist_ok=True)

    return data_dir


def get_data_dirs():
    dirs = [get_data_home()]
    xdg_data_dirs = os.getenv("XDG_DATA_DIRS") if os.getenv("XDG_DATA_DIRS") else "/usr/local/share/:/usr/share/"
    for d in xdg_data_dirs.split(":"):
        dirs.append(d)
    confirmed = []
    for d in dirs:
        if os.path.isdir(d):
            confirmed.append(d)

    return confirmed


def get_shell_data_dir():
    data_dir = ""
    home = os.getenv("HOME")
    xdg_data_home = os.getenv("XDG_DATA_HOME")

    if xdg_data_home:
        data_dir = os.path.join(xdg_data_home, "nwg-shell/")
    else:
        if home:
            data_dir = os.path.join(home, ".local/share/nwg-shell/")

    return data_dir


def temp_dir():
    if os.getenv("TMPDIR"):
        return os.getenv("TMPDIR")
    elif os.getenv("TEMP"):
        return os.getenv("TEMP")
    elif os.getenv("TMP"):
        return os.getenv("TMP")

    return "/tmp"


def data_home():
    if os.getenv("XDG_DATA_HOME"):
        return os.getenv("XDG_DATA_HOME")

    return os.path.join(os.getenv("HOME"), ".local/share")


def get_theme_names():
    theme_dirs = []
    for d in get_data_dirs():
        p = os.path.join(d, "themes")
        if os.path.isdir(p):
            theme_dirs.append(p)

    home = os.getenv("HOME")
    if home:
        p = os.path.join(home, ".themes")
        if os.path.isdir(p):
            theme_dirs.append(p)

    names = []
    exclusions = ["Default", "Emacs"]
    for d in theme_dirs:
        for item in os.listdir(d):
            if os.path.isdir(os.path.join(d, item)) and item not in exclusions:
                content = os.listdir(os.path.join(d, item))
                for name in content:
                    if name.startswith("gtk-"):
                        if item not in names:
                            names.append(item)
                            break
    names.sort()
    return names


def has_dirs(path):
    for item in os.listdir(path):
        if os.path.isdir(os.path.join(path, item)):
            return True
    return False


def get_theme_name(path):
    for item in os.listdir(path):
        if item == "index.theme":
            lines = load_text_file(os.path.join(path, item)).splitlines()
            for line in lines:
                if line.startswith("Name="):
                    return line.split("=")[1].strip()
    return None


def get_icon_themes():
    # In contrary to the get_theme_names() function, this time we need as well the theme name, as its folder name,
    # as we select icon themes by their folder names, and display names may be different. Odd, isn't it?
    icon_dirs = []
    for d in get_data_dirs():
        p = os.path.join(d, "icons")
        if os.path.isdir(p):
            icon_dirs.append(p)

    home = os.getenv("HOME")
    if home:
        p = os.path.join(home, ".icons")
        if os.path.isdir(p):
            icon_dirs.append(p)

    names = {}
    exclusions = ["default", "hicolor", "locolor"]
    for d in icon_dirs:
        for item in os.listdir(d):
            p = os.path.join(d, item)
            if item not in exclusions and os.path.isdir(p) and has_dirs(p):
                name = get_theme_name(os.path.join(d, item))
                if name:
                    names[name] = item
    return names


def init_files(src_dir, dst_dir, overwrite=False):
    running_hyprland = os.getenv("HYPRLAND_INSTANCE_SIGNATURE")
    src_files = os.listdir(src_dir)
    for file in src_files:
        if os.path.isfile(os.path.join(src_dir, file)):
            if overwrite or not os.path.isfile(os.path.join(dst_dir, file)):
                if "hyprland" not in file or running_hyprland:
                    copy2(os.path.join(src_dir, file), os.path.join(dst_dir, file))
                    print("Copying default file to '{}'".format(
                        os.path.join(dst_dir, file)))


def list_outputs():
    outputs = []
    try:
        from i3ipc import Connection
    except ModuleNotFoundError:
        print("'python-i3ipc' package required on sway, terminating")
        sys.exit(1)

    i3 = Connection()
    tree = i3.get_tree()
    for item in tree:
        if item.type == "output" and not item.name.startswith("__"):
            outputs.append(item.name)

    return outputs


def h_list_monitors():
    reply = hyprctl("j/monitors")
    try:
        outputs = []
        for item in json.loads(reply):
            if "name" in item:
                outputs.append(item["name"])
        return outputs
    except Exception as e:
        eprint(e)
        return []


def list_inputs_by_type(input_type=""):
    inputs = []
    try:
        from i3ipc import Connection
    except ModuleNotFoundError:
        print("'python-i3ipc' package required on sway, terminating")
        sys.exit(1)

    i3 = Connection()
    all_inputs = i3.get_inputs()
    for i in all_inputs:
        if i.type == input_type or not input_type:
            inputs.append(i.identifier)

    return inputs


def h_list_devices_by_type(device_type):
    reply = hyprctl("j/devices")
    devices = json.loads(reply)
    if devices and device_type in devices:
        return devices[device_type]

    return {}


def get_lat_lon():
    eprint("Checking location for night light settings")
    # Placeholder in case something goes wrong
    tz, lat, lon = "Warsaw", 52.2322, 20.9841

    lines = get_command_output("timedatectl show")
    if lines:
        for line in lines:
            if line.startswith("Timezone="):
                tz = line.split("=")[1].split("/")[-1]
        try:
            geolocator = Nominatim(user_agent="my_request")
            location = geolocator.geocode(tz)
            lat = round(location.latitude, 5)
            lon = round(location.longitude, 5)
        except:
            eprint(f"Couldn't find coordinates for {tz}.")

    eprint(f"Setting coordinates: {tz}, {lat}, {lon}")
    return tz, lat, lon


def is_command(cmd):
    return shutil.which(cmd) is not None


def gtklock_module_path(module_name):
    paths = ["/usr/lib/gtklock/{}-module.so".format(module_name), "/usr/local/lib/gtklock/{}-module.so".format(module_name)]
    for p in paths:
        if os.path.isfile(p):
            return p

    return ""


def get_command_output(command):
    try:
        return subprocess.check_output(command.split()).decode('utf-8').splitlines()
    except Exception as e:
        print("get_command_output() {}".format(e), file=sys.stderr)
        return []


def load_json(path):
    try:
        with open(path, 'r') as f:
            return json.load(f)
    except Exception as e:
        eprint("Error loading json: {}".format(e))
        return None


def save_json(src_dict, path, en_ascii=True):
    with open(path, 'w') as f:
        json.dump(src_dict, f, indent=2, ensure_ascii=en_ascii)


def load_text_file(path):
    try:
        with open(path, 'r') as file:
            data = file.read()
            return data
    except Exception as e:
        print(e)
        return None


def save_string(string, file):
    try:
        file = open(file, "wt")
        file.write(string)
        file.close()
    except:
        print("Error writing file '{}'".format(file))


def save_list_to_text_file(data, file_path):
    text_file = open(file_path, "w")
    for line in data:
        text_file.write(line + "\n")
    text_file.close()


def list_background_dirs():
    files_in_main = False
    paths = []
    main = "/usr/share/backgrounds"
    items = os.listdir(main)
    for item in items:
        try:
            p = os.path.join(main, item)
            if os.path.isdir(p) and os.listdir(p):
                paths.append(p)
            else:
                files_in_main = True
        except Exception as e:
            print(e)
    if files_in_main:
        paths.insert(0, main)

    return paths


def notify(summary, body, timeout=3000):
    cmd = "notify-send '{}' '{}' -i /usr/share/pixmaps/nwg-shell-config.svg -t {}".format(summary, body, timeout)
    subprocess.call(cmd, shell=True)


def check_key(dictionary, key, default_value):
    """
    Adds a key w/ default value if missing from the dictionary
    """
    if key not in dictionary:
        dictionary[key] = default_value


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def load_shell_data():
    shell_data_file = os.path.join(get_shell_data_dir(), "data")
    shell_data = load_json(shell_data_file)
    defaults = {
        "installed-version": get_shell_version(),
        "updates": [],
        "interface-locale": "",
        "autotranslated": False
    }
    if not shell_data:
        shell_data = defaults
        save_json(shell_data, shell_data_file)
        eprint("ERROR: '{}' file not found or corrupted. Initializing from defaults.".format(shell_data_file))
        eprint("The update history has been lost!")

    # Upgrade v0.2.5 -> v0.3.0
    # 1. If the "last-upgrade" key found, let's use the value as the "installed-version".
    # 2. We no longer need the pre-v0.3.0 "last-upgrade" key: delete it if found.
    if "last-upgrade" in shell_data:
        shell_data["installed-version"] = shell_data["last-upgrade"]
        del shell_data["last-upgrade"]
        save_json(shell_data, shell_data_file)

    for key in defaults:
        if key not in shell_data:
            shell_data[key] = defaults[key]

    return shell_data


def get_shell_version():
    lines = subprocess.check_output("nwg-shell -v".split()).decode('utf-8').splitlines()
    return lines[0].split()[2]


def playerctl_metadata():
    try:
        return subprocess.check_output("playerctl metadata 2>&1", shell=True).decode("utf-8").strip()
    except subprocess.CalledProcessError:
        return ""


def is_newer(string_new, string_existing):
    """
    Compares versions in format 'major.minor.patch' (just numbers allowed).
    :param string_new: new version to compare with existing one
    :param string_existing: existing version
    :return: True if new is newer then existing
    """
    new = major_minor_path(string_new)
    existing = major_minor_path(string_existing)
    if new and existing:
        if new[0] > existing[0]:
            return True
        elif new[1] > existing[1] and new[0] >= existing[0]:
            return True
        elif new[2] > existing[2] and new[0] >= existing[0] and new[1] >= existing[1]:
            return True
        else:
            return False
    else:
        return False


def major_minor_path(string):
    parts = string.split(".")
    if len(parts) != 3:
        return None
    try:
        return int(parts[0]), int(parts[1]), int(parts[2])
    except:
        return None


def log_line(file, label, line):
    label.set_markup(label.get_text() + line)
    file.write(line)


def do_backup(btn, config_home_dir, data_home_dir, backup_configs, backup_data, entry_backup, voc):
    dest_file = entry_backup.get_text() + ".tar.gz"
    if dest_file:
        id_file = os.path.join(temp_dir(), "nwg-shell-backup-id")
        save_string(dest_file.split("/")[-1], id_file)
        try:
            tar = tarfile.open(dest_file, "w:gz")
            tar.add(id_file, "nwg-shell-backup-id")
            for item in backup_configs:
                for name in os.listdir(os.path.join(config_home_dir, item)):
                    tar.add(os.path.join(config_home_dir, item, name))

            for item in backup_data:
                for name in os.listdir(os.path.join(data_home_dir, item)):
                    tar.add(os.path.join(data_home_dir, item, name))
            tar.close()
            notify(voc["backup"], "{}".format(dest_file))
        except Exception as e:
            # notify(voc["backup"], e)
            eprint("Couldn't backup data {}".format(e))


def unpack_to_tmp(fcb, restore_btn, restore_warn, voc):
    unpack_to = os.path.join(temp_dir(), "nwg-shell-backup")
    if os.path.isdir(unpack_to):
        shutil.rmtree(unpack_to)
    os.mkdir(unpack_to)
    backup = fcb.get_file().get_path()
    try:
        file = tarfile.open(backup)
        file.extractall(unpack_to)
        id_file = os.path.join(unpack_to, "nwg-shell-backup-id")
        if os.path.isfile(id_file):
            print("Unpacked backup from '{}'".format(load_text_file(id_file)))
            restore_btn.show()
            restore_warn.show()
        else:
            eprint("'{}' file is not a valid nwg-shell backup".format(backup))
            restore_btn.hide()
            restore_warn.hide()
            notify(voc["backup"], "{} {}".format(backup.split("/")[-1], voc["backup-invalid-file"]), 3000)
    except Exception as e:
        eprint("'{}'".format(backup), e)


def unpack_from_path(b_path):
    unpack_to = os.path.join(temp_dir(), "nwg-shell-backup")
    if os.path.isdir(unpack_to):
        shutil.rmtree(unpack_to)
    os.mkdir(unpack_to)
    try:
        file = tarfile.open(b_path)
        file.extractall(unpack_to)
        id_file = os.path.join(unpack_to, "nwg-shell-backup-id")
        if os.path.isfile(id_file):
            print("Unpacked backup from '{}'".format(load_text_file(id_file)))
        else:
            eprint("'{}' file is not a valid nwg-shell backup".format(b_path))
    except Exception as e:
        eprint("'{}'".format(b_path), e)


def restore_from_tmp(btn, restore_warning_label, voc):
    # The source and destination $HOME paths may be different, if we're restoring on another machine or user
    # The function may be called from outside GUI (-b argument); `btn` and `restore_warning_label` will be None, if so.
    parent_dir = os.path.join(temp_dir(), "nwg-shell-backup", "home")
    src_dir = os.path.join(parent_dir, os.listdir(parent_dir)[0])
    try:
        shutil.copytree(src_dir, os.getenv("HOME"), dirs_exist_ok=True)
        if btn:
            btn.hide()
        if restore_warning_label:
            restore_warning_label.hide()
        notify("{}".format(voc["backup"]), voc["backup-restore-success"], 10000)
        print("Configs and data restored.")
    except Exception as e:
        # notify(voc["backup"], "{}".format(e))
        eprint("Couldn't restore data {}".format(e))


def hyprctl(cmd, buf_size=20480):
    # /tmp/hypr moved to $XDG_RUNTIME_DIR/hypr in #5788
    xdg_runtime_dir = os.getenv("XDG_RUNTIME_DIR")
    hypr_dir = f"{xdg_runtime_dir}/hypr" if xdg_runtime_dir and os.path.isdir(
        f"{xdg_runtime_dir}/hypr") else "/tmp/hypr"

    s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    s.connect(f"{hypr_dir}/{os.getenv('HYPRLAND_INSTANCE_SIGNATURE')}/.socket.sock")

    s.send(cmd.encode("utf-8"))
    output = s.recv(buf_size).decode('utf-8')
    s.close()

    return output


def bool2lower(bool_value):
    if bool_value:
        return "true"
    else:
        return "false"



#!/usr/bin/env python3

"""
This script displays a simple confirmation window and executes a command, if given.
Repository: https://github.com/nwg-piotr/nwg-shell-config
Project site: https://nwg-piotr.github.io/nwg-shell
Author's email: nwg.piotr@gmail.com
Copyright (c) 2021-2025 Piotr Miller
License: MIT
"""

import argparse
import locale
import os
import subprocess
import sys

import gi

gi.require_version('Gtk', '3.0')
try:
    gi.require_version('GtkLayerShell', '0.1')
except ValueError:
    raise RuntimeError('\n\n' +
                       'If you haven\'t installed GTK Layer Shell, you need to point Python to the\n' +
                       'library by setting GI_TYPELIB_PATH and LD_LIBRARY_PATH to <build-dir>/src/.\n' +
                       'For example you might need to run:\n\n' +
                       'GI_TYPELIB_PATH=build/src LD_LIBRARY_PATH=build/src python3 ' + ' '.join(sys.argv))

from gi.repository import Gtk, Gdk, GtkLayerShell

from nwg_shell_config.tools import get_config_home, load_shell_data, load_json, eprint

config_home = get_config_home()
dir_name = os.path.dirname(__file__)
voc = {}
args = None


def handle_keyboard(win, event):
    if event.type == Gdk.EventType.KEY_RELEASE:
        if event.keyval == Gdk.KEY_Escape:
            Gtk.main_quit()


def launch(*arguments):
    if args.cmd:
        eprint(f"Executing: {args.cmd}")
        subprocess.Popen('{}'.format(args.cmd), shell=True)
        Gtk.main_quit()
    else:
        eprint("No command provided")
        Gtk.main_quit()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-c",
                        "--cmd",
                        type=str,
                        default="",
                        help='Command to execute if "Yes" clicked')

    parser.add_argument("-d",
                        "--dict",
                        action="store_true",
                        help="list Dictionary for current $LANG")

    parser.add_argument("-i",
                        "--icon",
                        type=str,
                        default="dialog-question",
                        help="Icon name ('dialog-question' by default)")

    parser.add_argument("-l",
                        "--lang",
                        type=str,
                        default="",
                        help="force use of a certain Lang value (like 'en_US', 'pl_PL')")

    parser.add_argument("-p",
                        "--prompt",
                        type=str,
                        default="Did you forget to include a prompt?",
                        help="dialog window Prompt: string or a dictionary key")

    parser.add_argument("-t",
                        "--translations",
                        action="store_true",
                        help="list available Translations")

    parser.parse_args()
    global args
    args = parser.parse_args()
    eprint(f"args: {args}")

    if args.lang:
        lang = args.lang
    else:
        shell_data = load_shell_data()
        lang = shell_data["interface-locale"] if "interface-locale" in shell_data and shell_data[
            "interface-locale"] else locale.getlocale()[0]
    eprint(f"User locale: {lang}")

    langs_dir = os.path.join(dir_name, "dialog")
    translations = os.listdir(langs_dir)

    global voc
    voc = load_json(os.path.join(langs_dir, "en_US"))

    if lang not in translations:
        eprint(f"Translation into '{lang}' not found. en_US will be used.")
    else:
        eprint(f"Translation into '{lang}' found.")
        user_lang = load_json(os.path.join(langs_dir, lang))
        for key in voc:
            if key in user_lang:
                voc[key] = user_lang[key]

    if args.dict:
        print(f"Key: value pairs for lang '{lang}':")
        for key in voc:
            print(f"'{key}': '{voc[key]}'")

    if args.translations:
        print(f"Available translations:")
        for t in sorted(translations):
            print(t)

    if args.dict or args.translations:
        sys.exit(0)

    prompt = voc[args.prompt] if args.prompt in voc else args.prompt

    window = Gtk.Window.new(Gtk.WindowType.POPUP)
    window.set_size_request(200, 0)

    GtkLayerShell.init_for_window(window)
    GtkLayerShell.set_layer(window, GtkLayerShell.Layer.TOP)
    GtkLayerShell.set_exclusive_zone(window, 0)
    GtkLayerShell.set_keyboard_mode(window, GtkLayerShell.KeyboardMode.ON_DEMAND)

    window.connect('destroy', Gtk.main_quit)
    window.connect("key-release-event", handle_keyboard)

    vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 12)
    vbox.set_property("margin", 12)
    window.add(vbox)

    box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
    vbox.pack_start(box, False, False, 0)
    img = Gtk.Image.new_from_icon_name(args.icon, Gtk.IconSize.DIALOG)
    box.pack_start(img, True, True, 0)

    box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
    vbox.pack_start(box, False, False, 0)
    lbl = Gtk.Label.new(prompt)
    lbl.set_property("justify", Gtk.Justification.CENTER)
    box.pack_start(lbl, True, True, 0)

    box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
    vbox.pack_start(box, False, False, 0)
    btn = Gtk.Button.new_with_label(voc["no"])
    btn.connect("clicked", Gtk.main_quit)

    box.pack_start(btn, True, True, 0)
    btn_y = Gtk.Button.new_with_label(voc["yes"])
    box.pack_start(btn_y, True, True, 0)

    screen = Gdk.Screen.get_default()
    provider = Gtk.CssProvider()
    style_context = Gtk.StyleContext()
    style_context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
    css = b"""window { border: solid 1px; border-color: #000; border-radius: 5px }"""
    provider.load_from_data(css)

    window.show_all()

    btn_y.connect("clicked", launch)
    btn_y.grab_focus()

    Gtk.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
This script displays a window containing given icon and text, a closes it on timeout.
Repository: https://github.com/nwg-piotr/nwg-shell-config
Project site: https://nwg-piotr.github.io/nwg-shell
Author's email: nwg.piotr@gmail.com
Copyright (c) 2024-2025 Piotr Miller
License: MIT
"""

import argparse
import json
import os
import subprocess
import sys

import gi

gi.require_version('Gtk', '3.0')
try:
    gi.require_version('GtkLayerShell', '0.1')
except ValueError:
    raise RuntimeError('\n\n' +
                       'If you haven\'t installed GTK Layer Shell, you need to point Python to the\n' +
                       'library by setting GI_TYPELIB_PATH and LD_LIBRARY_PATH to <build-dir>/src/.\n' +
                       'For example you might need to run:\n\n' +
                       'GI_TYPELIB_PATH=build/src LD_LIBRARY_PATH=build/src python3 ' + ' '.join(sys.argv))

from gi.repository import Gtk, Gdk, GLib, GdkPixbuf, GtkLayerShell

from shutil import copy2
from psutil import process_iter

from nwg_shell_config.tools import get_config_home, load_json, eprint

config_home = get_config_home()
config_dir = os.path.join(config_home, 'nwg-hud')
dir_name = os.path.dirname(__file__)
settings = None
args = None


def handle_keyboard(win, event):
    if event.type == Gdk.EventType.KEY_RELEASE:
        if event.keyval == Gdk.KEY_Escape:
            Gtk.main_quit()


def create_pixbuf(icon_name, icon_size):
    # In case a full path was given
    if icon_name.startswith("/"):
        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(
            icon_name, icon_size, icon_size)
    else:
        icon_theme = Gtk.IconTheme.get_default()
        try:
            pixbuf = icon_theme.load_icon(icon_name, icon_size, Gtk.IconLookupFlags.FORCE_SIZE)
        except:
            pixbuf = icon_theme.load_icon("image-missing", icon_size, Gtk.IconLookupFlags.FORCE_SIZE)

    return pixbuf


def update_image(image, icon_name, icon_size):
    scale = image.get_scale_factor()
    icon_size *= scale
    pixbuf = create_pixbuf(icon_name, icon_size)
    surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, scale, image.get_window())
    image.set_from_surface(surface)


def list_outputs():
    outputs_dict = {}
    if os.getenv("SWAYSOCK"):
        try:
            from i3ipc import Connection
        except ModuleNotFoundError:
            print("'python-i3ipc' package required on sway, terminating")
            sys.exit(1)

        i3 = Connection()
        outputs = i3.get_outputs()
        for item in outputs:
            outputs_dict[item.name] = {"description": f"{item.make} {item.model} {item.serial}",
                                       "monitor": None}

    elif os.getenv('HYPRLAND_INSTANCE_SIGNATURE') is not None:
        cmd = "hyprctl -j monitors"
        result = subprocess.check_output(cmd, shell=True).decode("utf-8").strip()
        outputs = json.loads(result)
        for item in outputs:
            outputs_dict[item["name"]] = {"description": item["description"],
                                          "monitor": None}

    else:
        eprint("nwg-hud script only works on sway or Hyprland, terminating")
        sys.exit(1)

    monitors = []
    display = Gdk.Display.get_default()
    for i in range(display.get_n_monitors()):
        monitor = display.get_monitor(i)
        monitors.append(monitor)

    for key, monitor in zip(outputs_dict.keys(), monitors):
        outputs_dict[key]["monitor"] = monitor

    # map monitor descriptions to output names
    mon_desc2output_name = {}
    for key in outputs_dict:
        if "description" in outputs_dict[key]:
            mon_desc2output_name[outputs_dict[key]["description"]] = key

    return outputs_dict, mon_desc2output_name


def main():
    # disallow multiple instances
    for proc in process_iter():
        if "nwg-hud" in proc.name() and proc.pid != os.getpid():
            eprint(f"nwg-hud: running instance found, PID {proc.pid}, terminating")
            sys.exit(1)

    # initiate config files if not found
    if not os.path.isdir(config_dir):
        os.mkdir(config_dir)
    for filename in ["config.json", "style.css"]:
        file_path = os.path.join(config_dir, filename)
        if not os.path.isfile(os.path.join(config_dir, filename)):
            copy2(os.path.join(dir_name, "hud", filename), file_path)

    # load settings
    global settings
    settings = load_json(os.path.join(config_dir, "config.json"))

    parser = argparse.ArgumentParser()
    parser.add_argument("-i",
                        "--icon",
                        type=str,
                        default="",
                        help='Icon name or path')

    parser.add_argument("-k",
                        "--keyboard",
                        action="store_true",
                        default=False,
                        help='turn on Keyboard support (Esc to close)')

    parser.add_argument("-z",
                        "--icon_size",
                        type=int,
                        default=48,
                        help="icon siZe")

    parser.add_argument("-m",
                        "--message",
                        type=str,
                        default="HUD message",
                        help='Message text to display')

    parser.add_argument("-t",
                        "--timeout",
                        type=int,
                        default=1000,
                        help="window Timeout in milliseconds")

    parser.add_argument("-ha",
                        "--horizontal_alignment",
                        type=str,
                        default="center",
                        help="window Horizontal Alignment: 'left' or 'right', 'center' by default")

    parser.add_argument("-va",
                        "--vertical_alignment",
                        type=str,
                        default="center",
                        help="window Vertical Alignment: 'top' or 'bottom', 'center' by default")

    parser.add_argument("-r",
                        "--margin",
                        type=int,
                        default=0,
                        help="window maRgin in pixels")

    parser.add_argument("-o",
                        "--output",
                        type=str,
                        default="",
                        help="name of the Output to display HUD on")

    parser.parse_args()
    global args
    args = parser.parse_args()

    # arguments override settings, if given
    if args.icon:
        settings["icon"] = args.icon
    if args.icon_size not in [0, 48]:
        settings["icon_size"] = args.icon_size
    if args.message:
        settings["message"] = args.message
    if args.timeout not in [0, 1000]:
        try:
            settings["timeout"] = int(args.timeout)
        except Exception as e:
            eprint(e)
    if args.horizontal_alignment != "center":
        settings["horizontal-alignment"] = args.horizontal_alignment
    if args.vertical_alignment != "center":
        settings["vertical-alignment"] = args.vertical_alignment
    if args.margin:
        try:
            settings["margin"] = int(args.margin)
        except Exception as e:
            eprint(e)
    if args.output:
        settings["output"] = args.output

    # make sure we have all values
    defaults = {
        "icon": "",
        "icon-size": 48,
        "message": "",
        "timeout": 1000,
        "horizontal-alignment": "",
        "vertical-alignment": "",
        "margin": 0,
        "output": ""
    }
    for key in defaults:
        if key not in settings:
            settings[key] = defaults[key]

    window = Gtk.Window.new(Gtk.WindowType.POPUP)

    GtkLayerShell.init_for_window(window)
    GtkLayerShell.set_layer(window, GtkLayerShell.Layer.TOP)
    GtkLayerShell.set_exclusive_zone(window, 0)
    if args.keyboard:
        GtkLayerShell.set_keyboard_mode(window, GtkLayerShell.KeyboardMode.ON_DEMAND)
    GtkLayerShell.set_namespace(window, "nwg-hud")

    if settings["vertical-alignment"] == "top":
        GtkLayerShell.set_anchor(window, GtkLayerShell.Edge.TOP, 1)
    elif settings["vertical-alignment"] == "bottom":
        GtkLayerShell.set_anchor(window, GtkLayerShell.Edge.BOTTOM, 1)
    if settings["horizontal-alignment"] == "left":
        GtkLayerShell.set_anchor(window, GtkLayerShell.Edge.LEFT, 1)
    elif settings["horizontal-alignment"] == "right":
        GtkLayerShell.set_anchor(window, GtkLayerShell.Edge.RIGHT, 1)

    if settings["margin"] > 0:
        GtkLayerShell.set_margin(window, GtkLayerShell.Edge.TOP, settings["margin"])
        GtkLayerShell.set_margin(window, GtkLayerShell.Edge.BOTTOM, settings["margin"])
        GtkLayerShell.set_margin(window, GtkLayerShell.Edge.LEFT, settings["margin"])
        GtkLayerShell.set_margin(window, GtkLayerShell.Edge.RIGHT, settings["margin"])

    # assign to a monitor if output name or monitor description given
    if settings["output"]:
        outputs, mon_desc2output_name = list_outputs()
        monitor = None
        if settings["output"] in outputs and "monitor" in outputs[settings["output"]]:
            monitor = outputs[settings["output"]]["monitor"]
        elif settings["output"] in mon_desc2output_name:
            name = mon_desc2output_name[settings["output"]]
            monitor = outputs[name]["monitor"]
        else:
            eprint(f"Couldn't assign monitor to {settings['output']}")

        if monitor:
            GtkLayerShell.set_monitor(window, monitor)

    window.connect('destroy', Gtk.main_quit)
    window.connect("key-release-event", handle_keyboard)

    vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
    vbox.set_property("margin", 0)
    window.add(vbox)

    hbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 12)
    hbox.set_property("margin", 12)
    vbox.pack_start(hbox, False, False, 0)

    if settings["icon"]:
        img = Gtk.Image()
        update_image(img, settings["icon"], settings["icon-size"])
        hbox.pack_start(img, False, False, 0)

    lbl = Gtk.Label()
    lbl.set_markup(settings["message"])
    hbox.pack_start(lbl, True, True, 0)

    # apply styling
    screen = Gdk.Screen.get_default()
    provider = Gtk.CssProvider()
    style_context = Gtk.StyleContext()
    style_context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
    try:
        provider.load_from_path(os.path.join(config_dir, "style.css"))
    except Exception as e:
        eprint(e)
    css = provider.to_string().encode('utf-8')
    provider.load_from_data(css)

    window.show_all()

    GLib.timeout_add(settings["timeout"], Gtk.main_quit)

    Gtk.main()


if __name__ == "__main__":
    main()
